//+------------------------------------------------------------------+ 
//|   Tester-Optimizer - BB + CCI - for FX v1.0                      |
//|                                                                  | 
//|   Copyright  2014 / airquest@hotmail.com                        |
//|                                                                  |
//|   http://smart-trading.world-record.ch                           |  
//+------------------------------------------------------------------+ 

#property copyright   "Copyright  2014 // airquest // smart-trading.world-record.ch"
#property link        "http://smart-trading.world-record.ch"
#property description "Tester-Optimizer - BB + CCI - for FX"
#property version     "1.0"

// Explanations :
// - Calculates the Win Rate and Balance for a reversal when the candle breaks the bands.
// - Entry on open of next candle and exit on TP or SL.
// - For entry, CCI has to be OB or OS on the same candle.
// - No trades : 1) When there is already a trade open; 2) When both High and Low have broken the bands.

//+------------------------------------------------------------------+
//| Buffers                                                          |
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Gray
#property indicator_color2 Red
#property indicator_color3 Green
#property indicator_style1 STYLE_DOT
#property indicator_width2 2
#property indicator_width3 2

//+------------------------------------------------------------------+
//| Enumerations                                                     |
//+------------------------------------------------------------------+

enum ENUM_MODE
{
   tester    = 0,  // Strategy Tester
   optimizer = 1   // Strategy Optimizer
};

//+------------------------------------------------------------------+

enum ENUM_COMMENTS
{
   labels    = 0,  // Labels
   comments  = 1,  // Comments
};

//+------------------------------------------------------------------+

enum ENUM_PRICES
{
   close          = 0,  // Close
   high_low       = 1,  // High-Low
   median         = 2,  // Median
   typical        = 3,  // Typical
   weighted_close = 4,  // Weighted Close
};

//+------------------------------------------------------------------+

enum ENUM_LIMIT_MODE
{
   max_of_selected_mode = 0,  // Max of Selected Mode
   default_settings     = 1,  // Max of Default Settings
   opti_settings        = 2,  // Max of Optimization Settings
};

//+------------------------------------------------------------------+
//| External variables                                               |
//+------------------------------------------------------------------+


input ENUM_MODE          IndicatorMode   = tester;
extern bool              ShowComments    = True;
input ENUM_COMMENTS      CommentsMode    = 1;

extern string            _Tester         = "================== Tester ==================";
extern int               BBPeriod        = 20;
input ENUM_MA_METHOD     BBMethod        = MODE_SMA;
input ENUM_APPLIED_PRICE BBPrice         = PRICE_CLOSE;
extern double            BBDeviation     = 2.0;
extern int               BBShift         =  0;
input ENUM_PRICES        PriceForBreak   = high_low;

extern int               CCIPeriod       = 14;
input ENUM_APPLIED_PRICE CCIPrice        = PRICE_TYPICAL;
extern int               CCIOBOSLevel    = 150;

extern double            StopLoss        = 300;
extern double            TakeProfit      = 500;
extern double            Spread          = 10;
extern double            Lots            = 1.0;

extern string            _OthersTester            = "----------------- Other Tester Settings -----------------";
extern bool              DrawArrows               = True;
extern bool              DrawSLTP                 = True;
extern bool              DrawLines                = True;
extern bool              ShowArrowsOnEntryCandle  = False;
extern bool              ShowSLTPOnEntryCandle    = False;

extern string            _Optimizer      = "================= Optimizer =================";
extern string            _MM             = "------------------ Money Management ------------------";
extern double            StopLossMin     = 300;
extern double            StopLossMax     = 300;
extern double            StopLossStep    = 100;
extern double            TakeProfitMin   = 500;
extern double            TakeProfitMax   = 500;
extern double            TakeProfitStep  = 100;

extern string            _BB             = "---------------------- Bollinger Bands ----------------------";
extern int               BBPeriodMin     = 5;
extern int               BBPeriodMax     = 100;
extern int               BBPeriodStep    = 5;
input ENUM_MA_METHOD     BBMethodMin     = MODE_SMA;
input ENUM_MA_METHOD     BBMethodMax     = MODE_SMA;
input ENUM_APPLIED_PRICE BBPriceMin      = PRICE_CLOSE;
input ENUM_APPLIED_PRICE BBPriceMax      = PRICE_CLOSE;
extern double            BBDeviationMin  = 2.0;
extern double            BBDeviationMax  = 2.0;
extern double            BBDeviationStep = 0.2;

extern string            _CCI            = "-------------- Commodity Channel Index --------------";
extern int               CCIPeriodMin    = 14;
extern int               CCIPeriodMax    = 14;
extern int               CCIPeriodStep   =  1;

input ENUM_APPLIED_PRICE CCIPriceMin     = PRICE_TYPICAL;
input ENUM_APPLIED_PRICE CCIPriceMax     = PRICE_TYPICAL;

extern int               CCILevelMin     = 150;
extern int               CCILevelMax     = 150;
extern int               CCILevelStep    =  25;

extern string            _OtherCommonSettings     = "=========== Other Common Settings ============";
extern bool              AllowMultipleOpenTrades  = True;
extern int               BarsToCount              = 1000;
extern bool              UseAllBars               = True;
input ENUM_LIMIT_MODE    LimitMode                = opti_settings;
extern bool              RunOnlyOnce              = True;

extern string            _CSVFile                 = "---------------------------- CSV File ----------------------------";
extern bool              WriteCSVFile             = True;
extern bool              WriteCSVFileOnTesterMode = False;
extern string            NameOfFileToWrite        = "SA_FX";
extern bool              AddIndicatorIDToName     = True;
extern string            IndicatorID              = "BB+CCI";
extern bool              AddSymbolToName          = True;
extern bool              AddTimeFrameToName       = True;
extern bool              AddLocalTimeToName       = True;
extern bool              SoundAtTheEndOfOpti      = True;
extern string            SoundToPlay              = "alert2.wav";

//+------------------------------------------------------------------+
//| Internal variables                                               |
//+------------------------------------------------------------------+

double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];
double deviation;

double Loss[];
double Win[];
double Balance[];

double SL[];
double TP[];
double RecSL[];
double RecTP[];
double Direction[];
double FirstLossShift[];
double FirstWinShift[];

double PriceUp[];
double PriceDn[];

double Osc[];

bool   file;
string priceForBreak;
string fileName;
string ID,symbol,TFName,cTime;
string MessageAlert1;
string MessageAlert2;
string MessageAlert3;

int    limitBars;
bool   dontTrade;
double tickValue = MarketInfo(NULL,MODE_TICKVALUE);
string Line1,Line2,Line3,Line4,Line5;
int    _BBPeriod,_BBPrice,_BBMethod,_CCIPeriod,_CCIPrice,_CCIOBOSLevel;
double _BBDeviation,_StopLoss,_TakeProfit;
string _BBMethodString,_BBPriceString,_CCIPriceString;

static double TotalWin=0,TotalTrades=0,TotalBalance=0,WinRate=0;

//+------------------------------------------------------------------+
//| Initialization function                                          |
//+------------------------------------------------------------------+

int init()
{
   IndicatorDigits(Digits);
   IndicatorBuffers(16);

   SetIndexBuffer(0,MovingBuffer);  SetIndexLabel(0,"MA");
   SetIndexBuffer(1,UpperBuffer);   SetIndexLabel(1,"Upper Band");
   SetIndexBuffer(2,LowerBuffer);   SetIndexLabel(2,"Lower Band");

   SetIndexDrawBegin(0,BBPeriod+BBShift);    SetIndexShift(0,BBShift);
   SetIndexDrawBegin(1,BBPeriod+BBShift);    SetIndexShift(1,BBShift);
   SetIndexDrawBegin(2,BBPeriod+BBShift);    SetIndexShift(2,BBShift);

   SetIndexBuffer(3,Loss);             SetIndexLabel(3,"Loss");
   SetIndexBuffer(4,Win);              SetIndexLabel(4,"Win");
   SetIndexBuffer(5,Balance);          SetIndexLabel(5,"Balance");
   SetIndexBuffer(6,SL);               SetIndexLabel(6,"Stop Loss");
   SetIndexBuffer(7,TP);               SetIndexLabel(7,"Take Profit");
   SetIndexBuffer(8,RecSL);            SetIndexLabel(8,"Rec SL");
   SetIndexBuffer(9,RecTP);            SetIndexLabel(9,"Rec TP");
   SetIndexBuffer(10,Direction);       SetIndexLabel(10,"Direction");
   SetIndexBuffer(11,FirstLossShift);  SetIndexLabel(11,"First Loss Shift");
   SetIndexBuffer(12,FirstWinShift);   SetIndexLabel(12,"First Win Shift");

   SetIndexBuffer(13,PriceUp);         SetIndexLabel(13,"Price High");
   SetIndexBuffer(14,PriceDn);         SetIndexLabel(14,"Price Low");  

   SetIndexBuffer(15,Osc);             SetIndexLabel(15,"Oscillator");

   if(IndicatorMode == tester)    { for(int i=3; i<16; i++) SetIndexStyle(i,DRAW_NONE); }
   if(IndicatorMode == optimizer) { for(    i=0; i<16; i++) SetIndexStyle(i,DRAW_NONE); }

   switch(PriceForBreak)
   {
      case 0: priceForBreak = "Close";          break;
      case 1: priceForBreak = "High-Low";       break;
      case 2: priceForBreak = "Median";         break;
      case 3: priceForBreak = "Typical";        break;
      case 4: priceForBreak = "Weigthed Close"; break;
   }

   if(IndicatorMode == tester)
   IndicatorShortName("Bollinger Bands ("+BBPeriod+","+BBPrice+","+BBMethod+","+BBDeviation+")/CCI ("+CCIPeriod+","+CCIPrice+","+CCIOBOSLevel+")");
   if(IndicatorMode == optimizer)
   IndicatorShortName("Bollinger Bands/CCI");

//+------------------------------------------------------------------+ Initialization of CSV file writing

   if((IndicatorMode == tester && WriteCSVFileOnTesterMode) || (IndicatorMode == optimizer && WriteCSVFile))
   {
      if(AddIndicatorIDToName)   ID     = IndicatorID+"-";                  else ID     = "";
      if(AddSymbolToName)        symbol = Symbol()+"-";                     else symbol = "";
      if(AddTimeFrameToName)     TFName = TimeFrameToString(Period())+"-";  else TFName = "";
      if(AddLocalTimeToName)     cTime  = GetTime()+"-";                    else cTime  = "";

      fileName = ID+symbol+TFName+cTime+NameOfFileToWrite+".csv";

      if(FileIsExist(fileName)) FileDelete(fileName);

      file = FileOpen(fileName,FILE_CSV|FILE_READ|FILE_WRITE,',');

      if(!file)
      {
         Alert("File "+fileName+" not found, the last error is ",GetLastError());
      }

      else
      {
         FileWrite(file,"Balance","Win Rate","Total Trades","Total Wins",
                        "Stop Loss","Take Profit",
                        "BB Period","BB Method","BB Price","BB Dev","Price For Break",
                        "CCI Period","CCI Price","CCI Level");
      }

      MessageAlert1 = "File created: '"+fileName+"', please check in your MQL4/Files folder.";
      MessageAlert2 = "Please remove or rename '"+fileName+"' or it will be rewritten on next run.";
      MessageAlert3 = "File may have not been written, please check last error: "+GetLastError();
   }

//+------------------------------------------------------------------+

   return(0);
}

//+------------------------------------------------------------------+
//| Deinitialization function                                        |
//+------------------------------------------------------------------+

int deinit()
{
   if(ShowComments)
   {
      if(CommentsMode == 0) DeleteLabels();
      if(CommentsMode == 1) Comment("");
   }
   if(DrawArrows)   DeleteArrows();
   if(DrawSLTP)     DeleteSLTP();
   if(DrawLines)    DeleteLines();
   if((IndicatorMode == tester && WriteCSVFileOnTesterMode) || (IndicatorMode == optimizer && WriteCSVFile))
   { if(file) FileClose(file); }

   return(0);
}

//+------------------------------------------------------------------+
//| Main calculation                                                 |
//+------------------------------------------------------------------+

int start()
{

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Avoid refreshing

   if(RunOnlyOnce) { static int CanRun = 0; if(CanRun == 0) CanRun = 1; else return(-1); }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Setting limit

   int counted_bars=IndicatorCounted();
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
   int i,limit;

   if     (LimitMode == max_of_selected_mode)
   {
      if(IndicatorMode == tester)
      { limit = MathMin(Bars-counted_bars,MathMin(Bars-1,MathMin(Bars-BBPeriod,Bars-CCIPeriod))); }
      if(IndicatorMode == optimizer)
      { limit = MathMin(Bars-counted_bars,MathMin(Bars-1,MathMin(Bars-MathMax(BBPeriod,BBPeriodMax),Bars-MathMax(CCIPeriod,CCIPeriodMax)))); }
   }
   else if(LimitMode == default_settings)
   { limit = MathMin(Bars-counted_bars,MathMin(Bars-1,MathMin(Bars-BBPeriod,Bars-CCIPeriod))); }
   else if(LimitMode == opti_settings)
   { limit = MathMin(Bars-counted_bars,MathMin(Bars-1,MathMin(Bars-MathMax(BBPeriod,BBPeriodMax),Bars-MathMax(CCIPeriod,CCIPeriodMax)))); }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Main calculation
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ 1/Strategy Tester
//+------------------------------------------------------------------+

   if(IndicatorMode == tester)
   {

//+------------------------------------------------------------------+ Beginning of tester cycle

      for(i=limit; i>=0; i--)
      {

//+------------------------------------------------------------------+ Settings buffers and prices

         Loss[i]           = 0;
         Win[i]            = 0;
         Balance[i]        = 0;
         SL[i]             = 0;
         TP[i]             = 0;
         RecTP[i]          = 0;
         RecSL[i]          = 0;
         Direction[i]      = 0;
         FirstLossShift[i] = 0;
         FirstWinShift[i]  = 0;

         switch(PriceForBreak)
         {
            case 0: { PriceUp[i] = Close[i];                            PriceDn[i] = Close[i];                          } break;
            case 1: { PriceUp[i] = High[i];                             PriceDn[i] = Low[i];                            } break;
            case 2: { PriceUp[i] = (High[i] + Low[i])/2;                PriceDn[i] = (High[i] + Low[i])/2;              } break;
            case 3: { PriceUp[i] = (High[i] + Low[i] + Close[i])/3;     PriceDn[i] = (High[i] + Low[i] + Close[i])/3;   } break;
            case 4: { PriceUp[i] = (High[i] + Low[i] + 2*Close[i])/4;   PriceDn[i] = (High[i] + Low[i] + 2*Close[i])/4; } break;
         }

//+------------------------------------------------------------------+ INDICATORS BUFFERS (Change them here)

         MovingBuffer[i] = iMA    (NULL,0,BBPeriod,0,BBMethod,BBPrice,i);
         deviation       = iStdDev(NULL,0,BBPeriod,0,MODE_SMA,BBPrice,i);
         UpperBuffer[i]  = MovingBuffer[i] + deviation * BBDeviation;
         LowerBuffer[i]  = MovingBuffer[i] - deviation * BBDeviation;
         Osc[i]          = iCCI(NULL,0,CCIPeriod,CCIPrice,i);

//+------------------------------------------------------------------+ Finding signals

         if(!UseAllBars && BarsToCount < limit) limitBars = BarsToCount;
         else                                   limitBars = limit;

         if(i <= limitBars)
         {

//+------------------------------------------------------------------+ Beginning of "if(i <= limitBars)" cycle

            dontTrade = PriceUp[i] > UpperBuffer[i] && PriceDn[i] < LowerBuffer[i];
   
            if     (!dontTrade && Osc[i] >  CCIOBOSLevel && PriceUp[i] > UpperBuffer[i])    // SELL SIGNALS (Change it here)
            {
               SL[i] = iOpen(NULL,0,i-1) + (StopLoss   / MathPow(10,Digits));
               TP[i] = iOpen(NULL,0,i-1) - (TakeProfit / MathPow(10,Digits));
               Direction[i] = -1;
            }
   
            else if(!dontTrade && Osc[i] < -CCIOBOSLevel && PriceDn[i] < LowerBuffer[i])    // BUY SIGNALS (Change it here)
            {
               SL[i] = iOpen(NULL,0,i-1) - (StopLoss   / MathPow(10,Digits));
               TP[i] = iOpen(NULL,0,i-1) + (TakeProfit / MathPow(10,Digits));
               Direction[i] =  1;
            }

//+------------------------------------------------------------------+ Finding if trade is a winner or loser

            if     (Direction[i] ==  1)
            {
               FirstLossShift[i] = CheckFirstLoserOrWinner(i,True, True, SL[i],TP[i]);
               FirstWinShift[i]  = CheckFirstLoserOrWinner(i,True, False,SL[i],TP[i]);
            }
   
            else if(Direction[i] == -1)
            {
               FirstLossShift[i] = CheckFirstLoserOrWinner(i,False,True, SL[i],TP[i]);
               FirstWinShift[i]  = CheckFirstLoserOrWinner(i,False,False,SL[i],TP[i]);
            }
   
            if     (FirstWinShift[i] > FirstLossShift[i])
            {
               Win[i] = 1; Loss[i] = 0; Balance[i] = TakeProfit*Lots*tickValue;
            }
   
            else if(FirstWinShift[i] < FirstLossShift[i])
            {
               Win[i] = 0; Loss[i] = 1; Balance[i] = -StopLoss *Lots*tickValue;
            }

//+------------------------------------------------------------------+ Deleting trades when a trade is already open

            if(!AllowMultipleOpenTrades)
            {
               if(TP[i] > 0) RecTP[i] = TP[i]; else if (RecTP[i+1] > 0) RecTP[i] = RecTP[i+1];
               if(SL[i] > 0) RecSL[i] = SL[i]; else if (RecSL[i+1] > 0) RecSL[i] = RecSL[i+1];
      
               if((High[i] < RecTP[i+1] && Low[i] > RecSL[i+1]) || (High[i] < RecSL[i+1] && Low[i] > RecTP[i+1]))
               {
                  RecTP[i] = RecTP[i+1];
                  RecSL[i] = RecSL[i+1];
               }
   
               if(TP[i] != RecTP[i] || SL[i] != RecSL[i]) 
               {
                  Win[i] = 0; Loss[i] = 0; Balance[i] = 0;
               }
            }

//+------------------------------------------------------------------+ Totals

            TotalWin     += Win[i];
            TotalTrades  += (Win[i] + Loss[i]);
            TotalBalance += Balance[i];
   
            if(TotalTrades == 0) WinRate = 0;
            else                 WinRate = (TotalWin * 100) / TotalTrades;

//+------------------------------------------------------------------+ Comments

            if(ShowComments)
            {
               Line1 = "Testing results for '"+priceForBreak+" breaks BB/CCI OB-OS/entry on next bar':";
               Line2 = "Period: "+BBPeriod+" / Method: "+BBMethod+" / Price: "+BBPrice+" / Dev: "+BBDeviation;
               Line3 = "CCI Period: "+CCIPeriod+" / CCI Price: "+CCIPrice+" / CCI Level: "+CCIOBOSLevel;
               Line4 = "Stop Loss: "+StopLoss+" / Take Profit: "+TakeProfit+" / Spread: "+Spread;
               Line5 = "Trades: "+TotalTrades+" / Wins: "+TotalWin+" / Win%: "+DoubleToString(WinRate,2)+"% / Balance: "+TotalBalance;

               if(CommentsMode == 0)
               {
                  color couleur = White;
                  int   corner  = 0;
                  int   x = 10;
                  int   y = 40;

                  DrawLabels(1,corner,x,y,   Line1,couleur);
                  DrawLabels(2,corner,x,y+10,Line2,couleur);
                  DrawLabels(3,corner,x,y+20,Line3,couleur);
                  DrawLabels(4,corner,x,y+30,Line4,couleur);
                  DrawLabels(5,corner,x,y+40,Line5,couleur);
               }

               if(CommentsMode == 1) Comment(Line1,"\n",Line2,"\n",Line3,"\n",Line4,"\n",Line5);
            }

//+------------------------------------------------------------------+ Objects

            if(DrawArrows)  ManageArrow(i);  else DeleteArrows();
            if(DrawSLTP)    ManageSLTP(i);   else DeleteSLTP();
            if(DrawLines)   ManageLines(i);  else DeleteLines();

//+------------------------------------------------------------------+ End of "if(i <= limitBars)" cycle

         }

//+------------------------------------------------------------------+ End of "for(i=limit; i>=0; i--)" cycle

      }

//+------------------------------------------------------------------+ Writing CSV file

      if(WriteCSVFileOnTesterMode)
      {
         switch(BBMethod)
         {
            case 0: _BBMethodString = "Simple";          break;
            case 1: _BBMethodString = "Exponential";     break;
            case 2: _BBMethodString = "Smoothed";        break;
            case 3: _BBMethodString = "Linear Weigthed"; break;
         }

         switch(BBPrice)
         {
            case 0: _BBPriceString = "Close";           break;
            case 1: _BBPriceString = "Open";            break;
            case 2: _BBPriceString = "High";            break;
            case 3: _BBPriceString = "Low";             break;
            case 4: _BBPriceString = "Median";          break;
            case 5: _BBPriceString = "Typical";         break;
            case 6: _BBPriceString = "Weigthed Close";  break;
         }

         switch(CCIPrice)
         {
            case 0: _CCIPriceString = "Close";           break;
            case 1: _CCIPriceString = "Open";            break;
            case 2: _CCIPriceString = "High";            break;
            case 3: _CCIPriceString = "Low";             break;
            case 4: _CCIPriceString = "Median";          break;
            case 5: _CCIPriceString = "Typical";         break;
            case 6: _CCIPriceString = "Weigthed Close";  break;
         }

         FileWrite(file,TotalBalance,DoubleToString(WinRate,2),TotalTrades,TotalWin,
                        StopLoss,TakeProfit,
                        BBPeriod,_BBMethodString,_BBPriceString,BBDeviation,priceForBreak,
                        CCIPeriod,_CCIPriceString,CCIOBOSLevel);

//+------------------------------------------------------------------+ Closing of CSV file and alerts

         if(file)
         {
            Alert(MessageAlert2);
            Alert(MessageAlert1);
            FileClose(file);
         }
      
         else
         {
            Alert(MessageAlert3);
         }
      }

//+------------------------------------------------------------------+ End of tester cycle

   }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ 2/Strategy Optimizer
//+------------------------------------------------------------------+

   if(IndicatorMode == optimizer)
   {

//+------------------------------------------------------------------+ Beginning of optimizer cycle

      for(_BBPeriod = BBPeriodMin; _BBPeriod <= BBPeriodMax; _BBPeriod += BBPeriodStep)
      {
         for(_BBPrice = BBPriceMin; _BBPrice <= BBPriceMax; _BBPrice += 1)
         {
            for(_BBMethod = BBMethodMin; _BBMethod <= BBMethodMax; _BBMethod += 1)
            {
               for(_BBDeviation = BBDeviationMin; _BBDeviation <= BBDeviationMax; _BBDeviation += BBDeviationStep)
               {
                  for(_CCIPeriod = CCIPeriodMin; _CCIPeriod <= CCIPeriodMax; _CCIPeriod += CCIPeriodStep)
                  {
                     for(_CCIPrice = CCIPriceMin; _CCIPrice <= CCIPriceMax; _CCIPrice += 1)
                     {
                        for(_CCIOBOSLevel = CCILevelMin; _CCIOBOSLevel <= CCILevelMax; _CCIOBOSLevel += CCILevelStep)
                        {
                           for(_StopLoss = StopLossMin; _StopLoss <= StopLossMax; _StopLoss += StopLossStep)
                           {
                              for(_TakeProfit = TakeProfitMin; _TakeProfit <= TakeProfitMax; _TakeProfit += TakeProfitStep)
                              {
                                 //_BBPeriod      = BBPeriod;         // Set these variables as default settings if you don't want to optimize them.
                                 //_BBPrice       = BBPrice;          // Also delete the for() cycle for the deleted parameter.
                                 //_BBMethod      = BBMethod;
                                 //_BBDeviation   = BBDeviation;
                                 //_CCIPeriod     = CCIPeriod;
                                 //_CCIPrice      = CCIPrice;
                                 //_CCIOBOSLevel  = CCIOBOSLevel;
                                 //_TakeProfit    = TakeProfit;
                                 //_StopLoss      = StopLoss;

                                 double _TotalWin = 0,_TotalTrades = 0,_TotalBalance = 0,_WinRate = 0;

//+------------------------------------------------------------------+ Beginning of of "for(i=limit; i>=0; i--)" cycle

                                 for(i=limit; i>=0; i--)
                                 {

//+------------------------------------------------------------------+ Settings buffers and prices

                                    Loss[i]           = 0;
                                    Win[i]            = 0;
                                    Balance[i]        = 0;
                                    SL[i]             = 0;
                                    TP[i]             = 0;
                                    RecTP[i]          = 0;
                                    RecSL[i]          = 0;
                                    Direction[i]      = 0;
                                    FirstLossShift[i] = 0;
                                    FirstWinShift[i]  = 0;

                                    switch(PriceForBreak)
                                    {
                                       case 0:
                                       { PriceUp[i] = Close[i];
                                         PriceDn[i] = Close[i];                           } break;
                                       case 1:
                                       { PriceUp[i] = High[i];
                                         PriceDn[i] = Low[i];                             } break;
                                       case 2:
                                       { PriceUp[i] = (High[i] + Low[i])/2;
                                         PriceDn[i] = (High[i] + Low[i])/2;               } break;
                                       case 3:
                                       { PriceUp[i] = (High[i] + Low[i] + Close[i])/3;
                                         PriceDn[i] = (High[i] + Low[i] + Close[i])/3;    } break;
                                       case 4:
                                       { PriceUp[i] = (High[i] + Low[i] + 2*Close[i])/4;
                                         PriceDn[i] = (High[i] + Low[i] + 2*Close[i])/4;  } break;
                                    }

//+------------------------------------------------------------------+ INDICATORS BUFFERS (Change them here)

                                    MovingBuffer[i] = iMA    (NULL,0,_BBPeriod,0,_BBMethod,_BBPrice,i);
                                    deviation       = iStdDev(NULL,0,_BBPeriod,0,MODE_SMA,_BBPrice,i);
                                    UpperBuffer[i]  = MovingBuffer[i] + deviation * _BBDeviation;
                                    LowerBuffer[i]  = MovingBuffer[i] - deviation * _BBDeviation;
                                    Osc[i]          = iCCI(NULL,0,_CCIPeriod,_CCIPrice,i);

//+------------------------------------------------------------------+ Finding signals

                                    if(!UseAllBars && BarsToCount < limit) limitBars = BarsToCount;
                                    else                                   limitBars = limit;

                                    if(i <= limitBars)
                                    {

//+------------------------------------------------------------------+ Beginning of "if(i <= limitBars)" cycle

                                       dontTrade = PriceUp[i] > UpperBuffer[i] && PriceDn[i] < LowerBuffer[i];
                              
                                       if     (!dontTrade && Osc[i] >  _CCIOBOSLevel && PriceUp[i] > UpperBuffer[i]) // SELL SIGNALS (Change it here)
                                       {
                                          SL[i] = iOpen(NULL,0,i-1) + (_StopLoss   / MathPow(10,Digits));
                                          TP[i] = iOpen(NULL,0,i-1) - (_TakeProfit / MathPow(10,Digits));
                                          Direction[i] = -1;
                                       }
                              
                                       else if(!dontTrade && Osc[i] < -_CCIOBOSLevel && PriceDn[i] < LowerBuffer[i])  // BUY SIGNALS (Change it here)
                                       {
                                          SL[i] = iOpen(NULL,0,i-1) - (_StopLoss   / MathPow(10,Digits));
                                          TP[i] = iOpen(NULL,0,i-1) + (_TakeProfit / MathPow(10,Digits));
                                          Direction[i] =  1;
                                       }

//+------------------------------------------------------------------+ Finding if trade is a winner or loser

                                       if     (Direction[i] ==  1)
                                       {
                                          FirstLossShift[i] = CheckFirstLoserOrWinner(i,True, True, SL[i],TP[i]);
                                          FirstWinShift[i]  = CheckFirstLoserOrWinner(i,True, False,SL[i],TP[i]);
                                       }
                              
                                       else if(Direction[i] == -1)
                                       {
                                          FirstLossShift[i] = CheckFirstLoserOrWinner(i,False,True, SL[i],TP[i]);
                                          FirstWinShift[i]  = CheckFirstLoserOrWinner(i,False,False,SL[i],TP[i]);
                                       }
                              
                                       if     (FirstWinShift[i] > FirstLossShift[i])
                                       {
                                          Win[i] = 1; Loss[i] = 0; Balance[i] = _TakeProfit*Lots*tickValue;
                                       }
                              
                                       else if(FirstWinShift[i] < FirstLossShift[i])
                                       {
                                          Win[i] = 0; Loss[i] = 1; Balance[i] = -_StopLoss *Lots*tickValue;
                                       }

//+------------------------------------------------------------------+ Deleting trades when a trade is already open

                                       if(!AllowMultipleOpenTrades)
                                       {
                                          if(TP[i] > 0) RecTP[i] = TP[i]; else if (RecTP[i+1] > 0) RecTP[i] = RecTP[i+1];
                                          if(SL[i] > 0) RecSL[i] = SL[i]; else if (RecSL[i+1] > 0) RecSL[i] = RecSL[i+1];
                                 
                                          if((High[i] < RecTP[i+1] && Low[i] > RecSL[i+1]) || (High[i] < RecSL[i+1] && Low[i] > RecTP[i+1]))
                                          {
                                             RecTP[i] = RecTP[i+1];
                                             RecSL[i] = RecSL[i+1];
                                          }
                              
                                          if(TP[i] != RecTP[i] || SL[i] != RecSL[i]) 
                                          {
                                             Win[i] = 0; Loss[i] = 0; Balance[i] = 0;
                                          }
                                       }

//+------------------------------------------------------------------+ Totals

                                       _TotalWin     += Win[i];
                                       _TotalTrades  += (Win[i] + Loss[i]);
                                       _TotalBalance += Balance[i];

                                       if(_TotalTrades == 0) _WinRate = 0;
                                       else                  _WinRate = (_TotalWin * 100) / _TotalTrades;

//+------------------------------------------------------------------+ End of "if(i <= limitBars)" cycle

                                    }

//+------------------------------------------------------------------+ End of "for(i=limit; i>=0; i--)" cycle

                                 }

//+------------------------------------------------------------------+ Writing CSV file

                                 if(WriteCSVFile)
                                 {
                                    switch(_BBMethod)
                                    {
                                       case 0: _BBMethodString = "Simple";          break;
                                       case 1: _BBMethodString = "Exponential";     break;
                                       case 2: _BBMethodString = "Smoothed";        break;
                                       case 3: _BBMethodString = "Linear Weigthed"; break;
                                    }

                                    switch(_BBPrice)
                                    {
                                       case 0: _BBPriceString = "Close";           break;
                                       case 1: _BBPriceString = "Open";            break;
                                       case 2: _BBPriceString = "High";            break;
                                       case 3: _BBPriceString = "Low";             break;
                                       case 4: _BBPriceString = "Median";          break;
                                       case 5: _BBPriceString = "Typical";         break;
                                       case 6: _BBPriceString = "Weigthed Close";  break;
                                    }

                                    switch(_CCIPrice)
                                    {
                                       case 0: _CCIPriceString = "Close";           break;
                                       case 1: _CCIPriceString = "Open";            break;
                                       case 2: _CCIPriceString = "High";            break;
                                       case 3: _CCIPriceString = "Low";             break;
                                       case 4: _CCIPriceString = "Median";          break;
                                       case 5: _CCIPriceString = "Typical";         break;
                                       case 6: _CCIPriceString = "Weigthed Close";  break;
                                    }

                                    FileWrite(file,_TotalBalance,DoubleToString(_WinRate,2),_TotalTrades,_TotalWin,
                                                   _StopLoss,_TakeProfit,
                                                   _BBPeriod,_BBMethodString,_BBPriceString,_BBDeviation,priceForBreak,
                                                   _CCIPeriod,_CCIPriceString,_CCIOBOSLevel);
                                 }

//+------------------------------------------------------------------+ End of optimizer loops

                              }
                           }
                        }
                     }
                  }
               }
            }
         }
      }

//+------------------------------------------------------------------+ Closing of CSV file and alerts

      if(WriteCSVFile)
      {
         if(file)
         {
            Alert(MessageAlert2);
            Alert(MessageAlert1);
            FileClose(file);
         }
      
         else
         {
            Alert(MessageAlert3);
         }
      }

      if(SoundAtTheEndOfOpti) PlaySound(SoundToPlay);

//+------------------------------------------------------------------+ End of optimizer cycle

   }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ End
//+------------------------------------------------------------------+

   return(0);
}

//+---------------------------------------------------------------------------------------+
//| Functions                                                                             |
//+---------------------------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Text Labels                                                      |
//+------------------------------------------------------------------+

void DrawLabels(int instance, int corner, int x, int y, string text, color couleur)
{
   string name = "Labels_"+instance;
   ObjectCreate (name,OBJ_LABEL,0,0,0);
   ObjectSetText(name,text,7,"Verdana",couleur);
   ObjectSet(name,OBJPROP_CORNER,corner);
   ObjectSet(name,OBJPROP_XDISTANCE,x);
   ObjectSet(name,OBJPROP_YDISTANCE,y);
}

//+------------------------------------------------------------------+

void DeleteLabels()
{
   string lookFor       = "Labels_";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//+------------------------------------------------------------------+
//| Arrows                                                           |
//+------------------------------------------------------------------+

void ManageArrow(int i)
{
   DeleteArrow(Time[i]);

   if(Direction[i] ==  1 && Win[i]  == 1)
   DrawArrow(i,Lime,233,1,False);
   if(Direction[i] ==  1 && Loss[i] == 1)
   DrawArrow(i,Red, 233,1,False);

   if(Direction[i] == -1 && Win[i]  == 1)
   DrawArrow(i,Lime,234,1,True);
   if(Direction[i] == -1 && Loss[i] == 1)
   DrawArrow(i,Red, 234,1,True);
}

//-------------------------------------------------------------------+

void DrawArrow(int i, color theColor, int theCode, int theWidth, bool up)
{
   string name = "Arrows_"+Time[i];
   double gap  = (3.0*iATR(NULL,0,20,i)/4.0) * 0.8;
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i]+ShowArrowsOnEntryCandle*(Period()*60),0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,theWidth);
         ObjectSet(name,OBJPROP_TIME1,Time[i]+ShowArrowsOnEntryCandle*(Period()*60));
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//-------------------------------------------------------------------+

void DeleteArrows()
{
   string lookFor       = "Arrows_";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//-------------------------------------------------------------------+

void DeleteArrow(datetime time)
{
   string lookFor = "Arrows_"+time; ObjectDelete(lookFor);
}

//+------------------------------------------------------------------+
//| SL and TP                                                        |
//+------------------------------------------------------------------+

void ManageSLTP(int i)
{
   DeleteSlTp(Time[i]);

   if(Win[i] != 0 || Loss[i] != 0)
   {
      DrawSLandTP(i,Blue,4,1,TP[i],"TP_");
      DrawSLandTP(i,Red, 4,1,SL[i],"SL_");
   }
}

//-------------------------------------------------------------------+

void DrawSLandTP(int i, color theColor, int theCode, int theWidth, double price, string whichOne)
{
   string name = whichOne+Time[i];

      ObjectCreate(name,OBJ_ARROW,0,Time[i]+ShowSLTPOnEntryCandle*(Period()*60),price);
      ObjectSet(name,OBJPROP_ARROWCODE,theCode);
      ObjectSet(name,OBJPROP_COLOR,theColor);
      ObjectSet(name,OBJPROP_WIDTH,theWidth);
      ObjectSet(name,OBJPROP_PRICE1,price);
      ObjectSet(name,OBJPROP_TIME1,Time[i]+ShowSLTPOnEntryCandle*(Period()*60));
}

//-------------------------------------------------------------------+

void DeleteSLTP()
{
   string lookFor1       = "SL_";
   string lookFor2       = "TP_";
   int    lookForLength1 = StringLen(lookFor1);
   int    lookForLength2 = StringLen(lookFor2);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength1) == lookFor1) ObjectDelete(objectName);
         if (StringSubstr(objectName,0,lookForLength2) == lookFor2) ObjectDelete(objectName);
   }
}

//+------------------------------------------------------------------+

void DeleteSlTp(datetime time)
{
   string lookFor1 = "SL_"+time; ObjectDelete(lookFor1);
   string lookFor2 = "TP_"+time; ObjectDelete(lookFor2);
}

//+------------------------------------------------------------------+
//| Lines                                                            |
//+------------------------------------------------------------------+

void ManageLines(int i)
{
   DeleteLine(Time[i]);

   if(Win[i] == 1)
   {
      drawLines(i,FirstWinShift[i],Blue,1,2,TP[i]);
   }

   if(Loss[i] == 1)
   {
      drawLines(i,FirstLossShift[i],Red,1,2,SL[i]);
   }
}

//-------------------------------------------------------------------+

void drawLines(int i, int shift, color theColor, int theWidth, int theStyle, double price)
{
   string name        = "Lines_"+Time[i];
   int    EntryCandle = 1;
      
      ObjectCreate(name,OBJ_TREND,0,Time[i]+EntryCandle*(Period()*60),Open[i-1],Time[shift],price);
      ObjectSet(name,OBJPROP_COLOR,theColor);
      ObjectSet(name,OBJPROP_WIDTH,theWidth);
      ObjectSet(name,OBJPROP_STYLE,theStyle);
      ObjectSet(name,OBJPROP_PRICE1,Open[i-1]);
      ObjectSet(name,OBJPROP_PRICE2,price);
      ObjectSet(name,OBJPROP_TIME1,Time[i]+EntryCandle*(Period()*60));
      ObjectSet(name,OBJPROP_TIME2,Time[shift]);
      ObjectSet(name,OBJPROP_RAY,False);
}

//-------------------------------------------------------------------+

void DeleteLines()
{
   string lookFor       = "Lines_";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//+------------------------------------------------------------------+

void DeleteLine(datetime time)
{
   string lookFor = "Lines_"+time; ObjectDelete(lookFor);
}

//+------------------------------------------------------------------+
//| TimeFrameToString                                                |
//+------------------------------------------------------------------+

string TimeFrameToString(int tf)
{
   string tfs;
   switch(tf)
   {
      case PERIOD_M1:     tfs = "M1";      break;
      case PERIOD_M5:     tfs = "M5";      break;
      case PERIOD_M15:    tfs = "M15";     break;
      case PERIOD_M30:    tfs = "M30";     break;
      case PERIOD_H1:     tfs = "H1";      break;
      case PERIOD_H4:     tfs = "H4";      break;
      case PERIOD_D1:     tfs = "D1";      break;
      case PERIOD_W1:     tfs = "W1";      break;
      case PERIOD_MN1:    tfs = "MN1";     break;
      default: tfs = "M" + DoubleToStr(tf,0);
   }
   return(tfs);
}

//+------------------------------------------------------------------+
//| GetTime                                                          |
//+------------------------------------------------------------------+

string GetTime()
{
   string time;
   int    replace;

   time    = TimeToString(TimeLocal(),TIME_DATE|TIME_MINUTES);
   replace = StringReplace(time,":","h");

   if(replace > 0) return(time);
   else            return(TimeLocal());
}

//+------------------------------------------------------------------+
//| CheckFirstLoserOrWinner                                          |
//+------------------------------------------------------------------+

int CheckFirstLoserOrWinner(int i, bool up, bool loss, double Sl, double Tp)
{
   int k,pre=0,result=0;

   for(k=i-1; k>=0; k--)
   {
      if(up)
      {
         if(loss) { if(iHigh(NULL,0,k) >= Sl                                 && iLow(NULL,0,k) <= Sl) { pre = k; if(pre > result) result = pre; } }
         else     { if(iHigh(NULL,0,k) + (Spread / MathPow(10,Digits)) >= Tp && iLow(NULL,0,k) <= Tp) { pre = k; if(pre > result) result = pre; } }
      }

      else
      {
         if(loss) { if(iHigh(NULL,0,k) + (Spread / MathPow(10,Digits)) >= Sl && iLow(NULL,0,k) <= Sl) { pre = k; if(pre > result) result = pre; } }
         else     { if(iHigh(NULL,0,k) >= Tp && iLow(NULL,0,k) + (Spread / MathPow(10,Digits)) <= Tp) { pre = k; if(pre > result) result = pre; } }
      }
   }

   return(result);
}

//+------------------------------------------------------------------+
//| The End                                                          |
//+------------------------------------------------------------------+